/**
 * @file
 * test.adjustColorBrightness node implementation.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

#include "VuoImageRenderer.h"

VuoModuleMetadata({
                     "title" : "Letter Box Filter",
                     "description" : "Letter box effect for framing the bottom and sides of an image. 2014, George Toledo.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoImageRenderer"
					 ],
					 "node": {
						 "isInterface" : false
					 }
				 });

#include "node.h"

static const char * fragmentShaderSource = VUOSHADER_GLSL_SOURCE(120,
	// Inputs

	varying vec4 fragmentTextureCoordinate;
    uniform sampler2D texture;
    uniform float fWidth;
    uniform float fHeight;
   // uniform float amount;
    uniform float topBorder;
    uniform float sideBorder;

    uniform vec4 color;
    void main()
    {
    vec4 tex=texture2D(texture,fragmentTextureCoordinate.xy);

        // Findout X , Y index of incoming pixel
        // from its texture coordinate.
        float fXIndex =fragmentTextureCoordinate.x * fWidth;
        float fYIndex =fragmentTextureCoordinate.y * fHeight;

        /* image boundaries Top, Bottom, Left, Right pixels*/
        if( ! ( fYIndex < topBorder || fYIndex > fHeight - topBorder ||
                fXIndex < sideBorder || fXIndex > fWidth - sideBorder )){

        gl_FragColor =tex;}
        else{
        gl_FragColor=color;}
    }
);
struct nodeInstanceData
{
	VuoShader shader;
	VuoGlContext glContext;
	VuoImageRenderer imageRenderer;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData * instance = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));
	VuoRegister(instance, free);

	instance->glContext = VuoGlContext_use();

	instance->shader = VuoShader_make("Adjust Color Brightness Shader");
	VuoShader_addSource(instance->shader, VuoMesh_IndividualTriangles, NULL, NULL, fragmentShaderSource);
	VuoRetain(instance->shader);

	instance->imageRenderer = VuoImageRenderer_make(instance->glContext);
	VuoRetain(instance->imageRenderer);

	return instance;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) instance,
		VuoInputData(VuoImage) image,
        VuoInputData(VuoColor,{"default":{"r":0,"g":0,"b":0,"a":1}}) Color,

        VuoInputData(VuoReal, {"default":1,"suggestedStep":1.0}) xBorder,
        VuoInputData(VuoReal, {"default":1,"suggestedStep":1.0}) yBorder,


		VuoOutputData(VuoImage) adjustedImage
)
{
	if (! image)
		return;

	// Feed parameters to the shader.
    VuoPoint4d col = { Color.r, Color.g, Color.b, Color.a };

	VuoShader_setUniform_VuoImage((*instance)->shader, "texture", image);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "sideBorder",     xBorder);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "topBorder",   yBorder);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "fWidth",   image->pixelsWide);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "fHeight",   image->pixelsHigh);
    VuoShader_setUniform_VuoPoint4d((*instance)->shader, "color", col);

	// Render.
	*adjustedImage = VuoImageRenderer_draw((*instance)->imageRenderer, (*instance)->shader, image->pixelsWide, image->pixelsHigh, VuoImage_getColorDepth(image));
}

void nodeInstanceFini(VuoInstanceData(struct nodeInstanceData *) instance)
{
	VuoRelease((*instance)->shader);
	VuoRelease((*instance)->imageRenderer);
	VuoGlContext_disuse((*instance)->glContext);
}
